# -*- encoding: UTF-8 -*-
'''
Created on 5 avr. 2010

@author: thierry
'''

import sys, os

import caritang.core.archiver
import caritang.core.source
import caritang.core.storage

from caritang.common import version 

from optparse import OptionParser


version.getInstance().submitRevision("$Revision: 34 $")


def replace_special_character(a_string):
    return a_string.replace("/","_").replace(".","") # to be modified if needed

def backupDir(login, password, folder, remove=False, flatten=False, verbose=False, *dirs ):
    '''
    Backup one or more given directories
    '''
   
    
    if flatten :
        for dir in dirs: 
            # generate as many remote directory as subfolder in local directory
            for root, dir_list, files in os.walk(dir) :
                folder_name = folder
                extra = root.partition(dir)[2] # part after dir
                if len(extra) > 0 :
                    if folder_name is None:
                        folder_name = replace_special_character(extra)
                    else :
                        folder_name = folder_name + "_" + replace_special_character(extra)                
                anArchiver = caritang.core.archiver.Archiver()
                anArchiver.addSource(caritang.core.source.Directory(root,recursive=False))
                docsStorage= caritang.core.storage.GoogleDocs(login, password, folder=folder_name)
                mediaStorage = caritang.core.storage.Picassa(login, password, album=folder_name)
                raidStorage = caritang.core.storage.AggregatorStorage()
                raidStorage.addStorage(mediaStorage)
                raidStorage.addStorage(docsStorage)
                anArchiver.addStorage(raidStorage)    
                if verbose :
                    anArchiver.save(progress_listener=caritang.core.archiver.verboseProgressListener())
                else:
                    anArchiver.save()
 
    else :
        anArchiver = caritang.core.archiver.Archiver() 
        for dir in dirs:
            anArchiver.addSource(caritang.core.source.Directory(dir))    
        docsStorage= caritang.core.storage.GoogleDocs(login, password, folder=folder)
        mediaStorage = caritang.core.storage.Picassa(login, password, album=folder)
        raidStorage = caritang.core.storage.AggregatorStorage()
        raidStorage.addStorage(mediaStorage)
        raidStorage.addStorage(docsStorage)
        anArchiver.addStorage(raidStorage)    
        if verbose :
            anArchiver.save(progress_listener=caritang.core.archiver.verboseProgressListener())
        else:
            anArchiver.save()

    

    
def run():
   versionManager = version.getInstance()
   usage = "%prog -l <login> -p <pasword>  [-d <destination>] [-r] [-f] directory ..."
   str_version = "%prog " + versionManager.getVersion() + "(" + versionManager.getRevision() + ")"
   parser = OptionParser(usage=usage, version=str_version)

   parser.add_option("-l","--login",action="store", dest="login", help="the login of the user without the @gmail")
   parser.add_option("-p","--password",action="store", dest="password", help="the password for the given login")
   parser.add_option("-d","--destination",action="store", dest="folder", help="the rmote folder or album name")
   parser.add_option("-r","--remove",action="store_true", dest="remove",  default=False, help="remove local file when uploaded")
   parser.add_option("-f","--flatten",action="store_true", dest="flatten", default=False, help="subfolder are flattened to folder with name prefixed by given -d option value if any")
   parser.add_option("-v","--verbose",action="store_true", dest="verbose", default=False, help="print verbose output of saving progress")   
   (options, args) = parser.parse_args()
   
   backupDir(options.login, options.password, options.folder, options.remove, options.flatten, options.verbose, *args)



if __name__ == '__main__':
    argline = sys.argv
    run()



# -*- encoding: UTF-8 -*-
'''
Created on 5 avr. 2010

@author: thierry
'''

import os.path
import hashlib

import storage

from caritang.common import version 

version.getInstance().submitRevision("$Revision: 41 $")

SAVED_SUCCESS = 0
SAVED_ALREADY = 1
SAVED_FAILURE = 2


class ProgressListener(object):
    '''
    Interface for progress listener
    '''
    def startSaveDocItem(self, docItem):
         raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def endSaveDocItem(self, docItem, storage, status):
         raise RuntimeError(NOT_YET_IMPLEMENTED)
       
class NoOpProgressListener(ProgressListener):

    def startSaveDocItem(self, docItem):
        pass


    def endSaveDocItem(self, docItem, storage, status):
        pass

class verboseProgressListener(ProgressListener):

    def startSaveDocItem(self, docItem):
        print "saving " + docItem.getFile() + " ...."



    def endSaveDocItem(self, docItem, storage, status):
        if status == SAVED_SUCCESS:
            print "ok in " + str(storage)
        elif status == SAVED_ALREADY:
            print "skipped"
        else : 
            print "FAILED in " + str(storage)

    pass

class Archiver(object):
    '''
    This class is responsible for upload documents to google docs account.
    '''
    
    

    def __init__(self):
        '''
        Constructor
        '''
        self.sources = []
        self.storages = []        

    def _get_caritang_storage_dir(self):
        '''
        Compute the file  system storage for caritang stuff
        '''
        storage = os.path.expanduser("~")
        storage = os.path.join(storage, ".caritang")
        return storage   
    
    def _get_storage_home(self, a_storage):
        storage = os.path.join(self._get_caritang_storage_dir(), a_storage.getName())
        return storage


    def _ensure_storage_home_exist(self, a_storage):
        storage = self._get_storage_home(a_storage)
        if not os.path.exists(storage):
            os.makedirs(storage)        
        return storage

            
    
    def addSource(self, aSource):
        self.sources.append(aSource) 
        
    def addStorage(self, aStorage):
        self.storages.append(aStorage)
    
    
    def save(self, resume=True, progress_listener=NoOpProgressListener()):
        '''
        save all source data to each storage. The default parameter resume is valued
        to True so the archiver attemps to resume the backup e.g. backup only new file.
        '''
        
        # ensure storage home exist for each storage
        for a_storage in self.storages:
            self._ensure_storage_home_exist(a_storage)
        
        for a_source in self.sources :
            for doc in a_source.list_document() :
                progress_listener.startSaveDocItem(doc)
                for a_storage in self.storages :
                    # check if the doc hasnot already been saved by this storage
                    if resume == False or not self._isMarkedAsSaved(doc, a_storage):
                        try :                                                                                                 
                            a_storage.store(doc)
                        except storage.CaritangStorageException, e:                        
                            progress_listener.endSaveDocItem(doc, a_storage, SAVED_FAILURE)
                        else:
                            # mark the doc as saved
                            self._markAsSaved(doc,a_storage)
                            progress_listener.endSaveDocItem(doc, a_storage, SAVED_SUCCESS)
                    else :
                        progress_listener.endSaveDocItem(doc, a_storage, SAVED_ALREADY)
                        
        
    
    
    def _isMarkedAsSaved(self,  a_docItem, a_storage):
        '''
        Test is a foc item has already been saved.
        '''
        storage = self._get_storage_home(a_storage)
        
        digest = self._computeDigestForDocItem(a_docItem)
        
        savedMarkup = os.path.join(storage, digest)
        
        return os.path.exists(savedMarkup)
      
    def _computeDigestForDocItem(self, a_docItem):   
        md5 = hashlib.md5(a_docItem.getFile())       
        digest = md5.hexdigest()
        return digest  
        
    def _markAsSaved(self,a_docItem, a_storage):
        '''
        Mark a docItem as saved.
        Further call to save with resume=True will not save the docItem again
        ''' 
        storage = self._get_storage_home(a_storage)
        
        digest = self._computeDigestForDocItem(a_docItem)
        
        savedMarkup = os.path.join(storage, digest)
        f = open(savedMarkup,"w")
        f.close()
        


# -*- encoding: UTF-8 -*-
'''
Created on 5 avr. 2010

@author: thierry
'''

import os.path

from caritang.common import version 

version.getInstance().submitRevision("$Revision: 33 $")


NOT_YET_IMPLEMENTED = "not yet implemented"

class DocSource(object):
    '''
    Interace for any document source. A document source may content many elements.
    '''
    
    def list_document(self):
        '''
        list all document from this source.
        Return an interator on DocItem
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)

class DocItem(object):
    '''
    Interface for a document item
    '''
    
    def getName(self):
        '''
        Return the name of the document
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getFile(self):
        '''
        Return the file url
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    

class Directory(DocSource):
    '''
    Local directory to backup.
    '''


    def __init__(self, directory, recursive=True, filter=None):
        '''
        Constructor
        '''
        self.directory = directory
        self.recursive = recursive
        self.filter = filter

    def list_document(self):
        for root, dirs, files in os.walk(self.directory) :
            if not self.recursive:
                del dirs[:]
            for file in files :
                yield FileDocItem(os.path.join(root, file))

class FileDocItem(DocItem):
    
    def __init__(self, filepath):
        self.filepath = filepath

    def getName(self):
        return os.path.basename(self.filepath)


    def getFile(self):
        return self.filepath
        
            


'''
Created on 5 avr. 2010

@author: thierry
'''

import os.path
import mimetypes
import gdata.docs
import gdata.docs.service

import gdata.photos.service
import gdata.media
import gdata.geo

from caritang.common import version 

version.getInstance().submitRevision("$Revision: 36 $")

NOT_YET_IMPLEMENTED = 'not yet implemented'

class DocStorage(object):
    '''
    Interface of DocStorage e.g. place where store document to.
    '''
    
    def store(self, doc):
        '''
        Store the docItem
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def canStore(self, doc):
        '''
        Test is this storage can store the document.
        The document is e DocItem
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getName(self):
        '''
        The name of the storage
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    

class CaritangStorageException(Exception):
    pass
            
    
class AggregatorStorage(DocStorage):
    '''
    This storage aggregate several storage to provide a seemless storage
    that can handle all documents of each aggregated storage.
    '''
    
    def __init__(self):
        self.storages = []

    def getName(self):
        return "aggregator"


    def store(self, doc):
        capables = filter(lambda(storage): storage.canStore(doc), self.storages)
        if len(capables) == 0:
            raise CaritangStorageException()
        storage = capables[0]
        storage.store(doc)

    def canStore(self, doc):
        for storage in self.storages:
            if storage.canStore(doc) :
                return True
        return False
    
    def addStorage(self, storage):
        self.storages.append(storage)
        
    
    
class Picassa(DocStorage):
    '''
    This is the Picassa implementation for a storage.
    '''
    
    mimetypes.add_type("video/mp4", ".mp4")
    gdata.photos.service.SUPPORTED_UPLOAD_TYPES = gdata.photos.service.SUPPORTED_UPLOAD_TYPES + ("mp4",)
    
    def __init__(self, login, password, album=None):
        '''
        Create a Picassastorage. If provided folder parameter is the folder
        where data will be uploaded.
        Note : login must contains the @gmail.com part        
        '''
        self.login = login
        
        self.gd_client = gdata.photos.service.PhotosService()
        self.gd_client.email = login
        self.gd_client.password = password
        self.gd_client.source = 'bressure.net-caritang-v1'
        self.gd_client.ProgrammaticLogin()
        
        self.album = album
        self.ensure_destination_folder()

    def canStore(self, doc):
        type = mimetypes.guess_type(doc.getName(),strict=False)[0]    
        if type == "image/jpeg" \
           or type == "video/mp4" :
            return True
        return False
           

    
    
    def ensure_destination_folder(self):
        '''
        Check existence of destination folder and create it if needed
        '''
        if self.album is not None:
            albums = self.gd_client.GetUserFeed()           
            remoteAlbum = filter(lambda(entry): entry.title.text == self.album, albums.entry)
            if len(remoteAlbum) == 0 :                
                self.album_entry = self.gd_client.InsertAlbum(title=self.album, summary='Album created by Caritang', access='protected')                
            else :
                self.album_entry = remoteAlbum[0]    
    
    def store(self, docItem):
        '''
        store the given doc to the storage
        '''      
        doc = docItem.getFile()
        type = mimetypes.guess_type(doc,strict=False)[0]        
        photoName = os.path.basename(doc)
        if self.album is None:
            photo = self.gd_client.InsertPhotoSimple('/data/feed/api/user/default/albumid/default', photoName, 
                                               'Uploaded with Caritang', doc, content_type=type)
        else :
            album_url = '/data/feed/api/user/%s/albumid/%s' % (self.login, self.album_entry.gphoto_id.text)
            photo = self.gd_client.InsertPhotoSimple(album_url, photoName, 
                            'Uploaded with Caritang', doc, content_type=type)                   

            
               
            
        
    def __str__(self):        
        if self.album is None :        
            return "Picassa for " + self.login
        else:
            return "Picassa for " + self.login + " in " + self.album

class GoogleDocs(DocStorage):
    '''
    This is the google docs implementation for a storage.
    '''


    def __init__(self, login, password, folder=None):
        '''
        Create a google doc storage. If provided folder parameter is the folder
        where data will be uploaded.
        Note : login must contains the @gmail.com part        
        '''
        self.login = login
        self.gd_client = gdata.docs.service.DocsService(source='bressure.net-caritang-v1')
        self.gd_client.ClientLogin(login, password)
        
        self.folder = folder
        self.ensure_destination_folder()

    def canStore(self, doc):
        return False # inhibate google doc 
    
    
    def ensure_destination_folder(self):
        '''
        Check existence of destination folder and create it if needed
        '''
        if self.folder is not None:
            q = gdata.docs.service.DocumentQuery(categories=['folder'], params={'showfolders': 'true'})
            feed = self.gd_client.Query(q.ToUri())
            remoteFolder = filter(lambda(entry): entry.title.text == self.folder, feed.entry)
            if len(remoteFolder) == 0 :
                self.folder_entry = self.gd_client.CreateFolder(self.folder)
            else :
                self.folder_entry = remoteFolder[0]    
   
    
    def store(self, docItem):
        '''
        store the given doc to the storage
        '''      
        doc = docItem.getFile()
        type = mimetypes.guess_type(doc,strict=False)[0]        
               
        ms = gdata.MediaSource(file_path=doc, content_type=type)
        if self.folder is None:
            entry = self.gd_client.Upload(ms, os.path.basename(doc))            
        else :
            entry = self.gd_client.Upload(ms, os.path.basename(doc), folder_or_uri=self.folder_entry)
        
               
            
        
    def __str__(self):
        if self.folder is None : 
            return "google docs for " + self.login
        else : 
            return "google docs for " + self.login + " in" + self.folder


# -*- encoding: UTF-8 -*-
'''
Created on 28 mars 2010

@author: thierry
'''



CURRENT_VERSION = "0.2.1"  # this must be modified before any release

def getInstance():
    return SINGLETON

class Version(object):
    '''
    Hold the revision of the application.
    '''        

    def __init__(self):
        '''
        Constructor
        '''
        self.revision = ""
        
    def submitRevision(self, rev):
        '''
        submit a new revision. If the revision is upper than
        the current revision, it become the new current revision
        '''
        
        if self.revision < rev:
            self.revision = rev
            
    def getRevision(self):
        '''
        Return the scm revision for this application
        '''
        return self.revision
    
    def getVersion(self):
        '''
        Return the marketing version for this application.
        '''
        return CURRENT_VERSION
    
SINGLETON = Version()  # object that track the revision of current applcation for issue tracking purpose

getInstance().submitRevision("$Revision: 43 $")



# -*- encoding: UTF-8 -*-
'''
Created on 5 avr. 2010

@author: thierry
'''
from distutils.core import setup
setup(name='caritang',
      version='0.2.1',
      package_dir = {'': 'src'},
      packages=['caritang',
                'caritang.common',
                'caritang.core',
                'caritang.gui'],               
      scripts=['scripts/caritang'],
      package_data={'caritang': ['*.png']},      
      author='Thierry Bressure',
      author_email='thierry@bressure.net',
      maintainer='Thierry Bressure',
      maintainer_email='caritang@bressure.net',
      url='http://blog.caritang.bressure.net',
      download_url='http://caritang.bressure.net',
      description='Caritang a simple backup application for N900 to google docs',
      long_description='Caritang let you to upload content of your N900 into a google docs account. You can specify which data to backup',
      classifiers=[
          'Development Status :: 2 - Pre-Alpha',
          "Environment :: Handhelds/PDA's",          
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',          
          'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python',
          'Topic :: System :: Archiving :: Backup'
          ],

      )


